% Main Matlab program for :
% Gapen, Michael T. and Thomas F. Cosimano, 
% "Sovling Ramsey Problems with Nonlinear Projection Methods," 
% Studies in Nonlinear Dynamics and Econometrics, Vol. 9, Spring
% 2005.
%
% Filename: ProjectionSNDE.m
%
% This program draws on the following Matlab subroutine m-files:
% chebnode.m : Creates n Chebyshev nodes in the [-1,1] interval. 
% csolvesimsnew.m : Nonlinear solver routine.
% hpfilter.m : The Hodrick-Prescott filter.
% projectionSNDE2.m : Creates the system of residual equations.
% hernodes.m : Computes the weights and nodes for Hermite-Polynomials.
% scaleup.m : Linearly rescales every element of the vector x from [-1,1] 
%    to  [xmin,xmax]
% mfun.m : Creates the bivariate Chebyshev Polynomial for the money function.
% hfun.m: Creates the bivariate Chebyshev Polynomial for the labor function.
% taufun.m : Creates the bivariate Chebyshev Polynomial the for tax function.
% lambdafun.m : Creates the bivariate Chebyshev Polynomial for the lambda function
%    in the government optimization problem.
% mpoly.m : Creates the bivariate Chebyshev Polynomial for money.
% hpoly.m : Creates the bivariate Chebyshev Polynomial for labor.
% taupoly.m : Creates the bivariate Chebyshev Polynomial for taxes.
% lambdapoly.m : Creates the bivariate Chebyshev Polynomial for lambda.
% chebpol.m : Create an ord-th order univariate Chebyshev Polynomial.
% rlquad5.m : real interest rate based on Gauss Hermite quadrature.
% tauquad5.m : Gauss Hermite quadrature routine
% c2tquad5.m : Gauss Hermite quadrature routine
% muquad5.m : Gauss Hermite quadrature routine
% muquad6.m : Gauss Hermite quadrature routine
% lambdaquad.m : Gauss Hermite quadrature routine
%
% Opening commands
% ====================================================================================
% (may want to list a GLOBAL statement, CLEAR ALL, and DIARY commands)

clear all;
global alpha beta a rhotheta rhog sigmatheta sigmag thetamax thetamin gmax gmin
global thetaord gord tnod gnod G0 theta0 Y maxiter coefftol gamma
global C1 C2 M mu G_Y C1_Y H g0 Kt D_Y X Ynet

% format long lists 15 digits, format short lists 5
format long;
%
start = 1;      % If start = 1 use initial guess. If = 0, use assigned values.
printer = 1;    % If printer = 1 then will print.  If = 0 then screen output supressed.
graph = 1;      % If graph = 1 then will plot graph. If = 0 then plotting suppressed.
solver = 0;     % If solver = 1 then proceed to solver.  If =0 then proceed to simulation.
test = 0;       % If test = 1 only run loop once.  If test = 0 call equation solver.
impulse = 1;   % If impulse = 1 then run impulse response.  If =0, then skip.
matrix = 0;     % If matrix = 1 then create matrix of shocks. If=0, then skip.
utilitycalc = 0; % If utilitycal=1 then compute utility. If =0, then skip.
simulate = 1;   % If simulate = 1 then full simulation.  If = 0 then no simulation.
vector = 0;     % If vector = 1. then compute new utilitylong vector. If = 0, then skip.
utilitylong = 0; % If utilitylong = 1, then long single vector. If =0 then skip.
%
% ====================================================================================
%
% Section 1.0 Calibration
% =======================
%
% Calibrate debt parameters
% -------------------------
%
D_Y= 0.49;	% Total Federal debt held by households / Net National Product = 0.49
BL_D=1; 			
%
% Display debt parameters for this case for the output file
%
if(printer),fprintf(1,'\n D/Y %g, BL/D %g',D_Y,BL_D),end;
%                     
% Calibrate model parameters and steady state
% -------------------------------------------
%
RL=1.0093;		    % Quarterly real interest rate
beta=1/RL;		    % Rate of time discount
rhotheta=0.95;		% Persistence of technology shock
sigmatheta=0.007;	% Standard deviation of technology shock 
rhog=0.96;			% Persistence of government spending shock
sigmag=0.021;		% Standard deviation of government spending shock
theta_ss=0;			% Steady state value of productivity shock
g_ss=0;				% Steady state value of government spending
theta0=theta_ss;	% Initial value for theta shock
g0=g_ss;		    % Initial value for spending shock
alpha=0.60;	        % Labor's share of income
H=0.31; 			% Fraction of time spent working
Y_K=0.0753; 		% Quarterly output to capital ratio
K_Y=1/(Y_K);        % Quarterly capital to output ratio
H_K=0.01343;    	% Quarterly labor to capital ratio
K=H/H_K;		    % Steady state capital stock
Kt=K^(1-alpha);     % Capital for the production function
Y=Kt*H^alpha;	    % Steady state level of income
delta = 0.0168;     % Quarterly rate of depreciation
Ynet = Y*(1-delta*K_Y); % Net National Product
%
BL_Ynet=BL_D*D_Y*Ynet;	% Stock of debt relative to net national product
BL_Y = BL_Ynet/Ynet;    % Debt as a percent of total income
D=Ynet*D_Y;             % Stock of total debt
mu=log(beta);   
C1_Ynet = 0.36;         % Assume C1/Y based on regression
M=0.07*Ynet;            % Steady state stock of money. Mbase/Ynet is 7 percent.
C1_M=C1_Ynet*(1/.07);   % C1/M = C1/Y*Y/M
G_Y = 0.18;             % G spending as a percent of net income
G=G_Y*Ynet;             % Steady state level of government spending
C1 = C1_Ynet*Ynet;      % Cash Good
X = delta*K;            % Investment
C2 = Y-C1-X-G;          % Credit Good
C1_C2=C1/C2;
a=(C1_C2)/(1+C1_C2);	% Relative importance of the cash good and credit good
				        % in the utility function
P = M/C1;               % Price level
gamma = 2.4735;         % Marginal disutility of supplying additional labor.
G0=G;					% Initial level for spending
%
% Important parameters for solution procedure and simulation routine
% ------------------------------------------------------------------
maxiter=5;  			% number of iterations for projection solution routine							 
coefftol=1e-7;			% Convergence criterion
nper = 500;            % number of periods for simulations. 
nsim = 1;              % number of simulations to compute sample statistics
%
% Calibrate algorithm parameters
% ------------------------------
% These are the n-thetas and n-gs in the model.  These determine
% the order of the polynomials for the policy functions in multiplicative fashion.
%
thetaord=2;		% Order of polynomial for productivity shock
gord=2;			% Order of polynomial for government shock
%
% These will be the m-thetas and m-gs in the model.  These determine the grid over which
% the model will be solved.  This is the coallocation portion of the solution procedure.
% The actual grid is not calculated here, but will be backed out later.  Since the Cheby
% polynomials act over [-1,1] we will use their optimal nodes with the min/max values to
% yield the grid over theta and g (their ranges will be [theta_a, theta_b], [g_a, g_b].
% Hence, only the number of gridpoints and the min/max values need to be specified.
%
thetagrid=thetaord+1;   % Number of gridpoints for productivity shock
ggrid=gord+1;			% Number of gridpoints for government spending shock

theta_min=-2.0; % Lowest gridpoint for theta in std deviations away from mean
theta_max=2.0;	% Highest gridpoint for theta in std deviations away from mean
g_min=-2.0;		% Lowest gridpoint for g in std deviations away from mean
g_max=2.0;		% Highest gridpoint for g in std deviations away from mean

thetamin=theta_min*sigmatheta;
thetamax=theta_max*sigmatheta;
gmin=g_min*sigmag;
gmax=g_max*sigmag;
%
% Specify the number of nodes for the Gauss-Hermite Quadrature routine.  These are 
% the r-thetas and r-gs in the model
%
tnod=11;
gnod=11;
%
% Section 2.0 Create the grid and state spaces
% =============================================
% This creates a grid over the possible realizations of the productivity and spending
% shocks in the model. Each possible productivity shock is paired with all possible 
% spending shocks.
%
% All gridpoints are stored in a matrix xgrid with dimensions (thetagrid*ggrid)*2.
% Note that the gridpoints will be the standard Chebyshev nodes which are in the
% interval [-1,1].  The Chebyshev nodes are chosen optimally rather than by even spaced
% intervals. A view of xgrid will show a grid from [-1,1] that is based on 
% the standard deviations specified above. The function scalup.m will scale 
% the value of the state variable given any point in [-1.1].
%
thetag = chebnode(thetagrid);
gg = chebnode(ggrid);
%
% The following loop combines the grids by pairing up each possible realization of theta
% and g on the grid. xgrid is a (thetagrid*grid) x 2 matrix.  These dimensions are then 
% stored in [dimgrid nvars] below.
%
for ti = 1:thetagrid;
   for gi = 1:ggrid;
      xgrid((ti-1)*ggrid+gi,:) = [thetag(ti) gg(gi)];
   end;
end;
%
% Compute the size of the polynomials
% -----------------------------------
% This process creates vectors of zeros equal to the number of coefficients needed
% for each polynomial.  Since there are two random variables, productivity and spending,
% the number of coefficients will be equal to the size of the gridspace over these.
%
lamsize=(thetaord+1)*(gord+1);
lam=zeros(lamsize,1);
%
psisize=(thetaord+1)*(gord+1);
psi=zeros(psisize,1);
%
omegasize=(thetaord+1)*(gord+1);
omega=zeros(omegasize,1);
%
phisize=(thetaord+1)*(gord+1);
phi=zeros(phisize,1);
%
% Initialize the coefficient vectors
% ----------------------------------
% The lam, psi, omega, and phi vectors above contain the coefficients to be solved for.
% As an initial guess, set the values below.  These are then stored in 
% a comprehensive coeffecient vector, coeffvec.  
%
% If start = 1 then guess the initial conditions. If start = 0, then use the
% assigned values below
%
if start
    lamguess=ones(lamsize,1);
    psiguess=ones(psisize,1);
    omegaguess=ones(omegasize,1);
    phiguess=ones(phisize,1);
    % calibrate
  %  if thetaord == 1;
        lam=lamguess/40; 
        psi=psiguess/4;  
        phi=phiguess/13;
        omega=omegaguess/20;  
    %end;
else
    %
    % The Low debt case
    if thetaord ==1 & D_Y==0.49;
        lam =[-0.42051813161970
   0.96449152204900
   0.34835464572657
  -0.75405758989611];
        phi =[0.38438668861555
  -0.00108787001043
  -0.07471315922017
   0.00078122885877];
        psi =[1.14608005489346
  -0.16519571022884
  -0.11173242077311
   0.14011940865206];
        omega =[0.05724290422851
   0.32378905749708
  -0.03995604491002
  -0.15297341581036];
   %
   % The Zero Debt Case 
    elseif thetaord ==1 & D_Y ==0;    
        lam = [-0.28759562990857
   0.78430842248426
   0.23229015268274
  -0.59603628623822];
        phi = [0.38682386115966
  -0.00000000000495
  -0.07599962890564
   0.00000000001307];
        psi = [1.00929999996711
   0.00000000000586
   0.00000000000560
   0.00000000002099];
        omega = [0.00404514034693
   0.37531697502065
  -0.00000000000521
  -0.19640016884790];
    %
    %The high debt case
    elseif thetaord ==1 & D_Y==2*0.49;
       lam =[-0.56108901604816
   1.15476093919528
   0.47183716251746
  -0.92183348604116];
        phi =[0.38169452944768
  -0.00193105458091
  -0.07321072118008
   0.00134330458651];
        psi =[1.29434062069875
  -0.34439175032154
  -0.23419230708287
   0.29348373653211];
        omega =[0.11478178318153
   0.26715446857740
  -0.08397677440744
  -0.10469853527031];
   %
   end;
%
end;
%
% Create the comprehensive coefficient vector
% -------------------------------------------
% Store the individual coefficient vectors into one comprehensive coeffecient vector.
% This is needed because the solver program will need to rever to a single stacked vector
% of residual functions and a single stacked vector of coefficients
%
coeffvec=[lam ; phi ; psi ; omega];
%
% Section 3.0 Call on the Equation Solver
% =======================================
%
% If test = 1, the residual vector will be produced, 
% but will not call the solver pogram.  
% If test = 0, the solver program is called. 
%
if test
    residvec=projectionSNDE2(coeffvec,xgrid);
    residvec;
    normresid=norm(residvec);
    if(printer),fprintf(1,'\n normresid %10g',normresid),end;
else
    if solver
    gradv=[];
    [coeff,rc] = csolvesimsnew('projectionSNDE2',coeffvec,xgrid,gradv,coefftol,maxiter);
    if(printer)
        lam=coeff(1:(thetaord+1)*(gord+1))
        phi=coeff(((thetaord+1)*(gord+1)+1):(2*(thetaord+1)*(gord+1)))
        psi=coeff((2*(thetaord+1)*(gord+1)+1):(3*(thetaord+1)*(gord+1)))
        omega=coeff((3*(thetaord+1)*(gord+1)+1):(4*(thetaord+1)*(gord+1)))            
    end
end
end
%
% Section 4.0 Create excel file of shocks
% =======================================
if matrix
    % Need to set matrix=1 above.
    %
    % Outer loop defines the number of simulations.
    %
    for si = 1:nsim
        %
        % The shock is drawn from a random number generator based on the 
        % calibrated shock values. Create a new vector of shocks for each
        % simulation. Each row will be a nper-long vector of shocks. The 
        % number of rows defines the number of simulations with nper
        % defining the number of periods in each simulation.
        %
        % Random number generator scaled by shocks.
        epsilon_theta = sigmatheta*randn(1,nper);
        epsilon_g = sigmag*randn(1,nper);
        % Define first observation
        theta_t(si,1) = rhotheta*theta0 + epsilon_theta(1);
        g_t(si,1) = rhog*g0 + epsilon_g(1);
        % Create a new vector of shocks for each simulation.
        % 
        for vi=2:nper
            theta_t(si,vi) = rhotheta*theta_t(si,vi-1) + epsilon_theta(vi);
            g_t(si,vi) = rhog*g_t(si,vi-1) + epsilon_g(vi);
        end;
        %
        % A(:,j) = the j-th column of A
        % A(i,:) = the i-th row of A
        %
        thetamatrix = theta_t;
        gmatrix = g_t;
        %
        save shockthetaedit thetamatrix;
        save shockgedit gmatrix;
    end;
    % 
end;
%
% Section 5.0 Simulation 
% ======================
if simulate
    % Outer loop defines the number of simulations. Inner loop defines 
    % the number of periods for each simulation.
    % Equal to nper as defined above for impulse response functions.
    %
    % Set initial shock to zero and compute initial values
    M0=M;
    Y0 = exp(theta0)*Kt*(hfun(exp(theta0),exp(g0),phi))^alpha;
    epsilon_theta(1)=0;
    epsilon_g(1)=0;
    theta_t(1) = rhotheta*theta0 + epsilon_theta(1);
    g_t(1) = rhog*g0 + epsilon_g(1);
    P0 = M/C1;
    X0=X;
    BLt0 = BL_D*D_Y*(Y0-X0);
    RLt0 = rlquad5(phi,psi);
    %
    for si = 1:nsim
        %
        % The shock is drawn from a random number generator based on the 
        % calibrated shock values. Create a new vector of shocks for each
        % simulation.
        %
        epsilon_theta(2:nper) = 0 + sigmatheta*randn(1,nper-1);
        epsilon_g(2:nper) = 0 + sigmag*randn(1,nper-1);
        %
        for vi=2:nper
            theta_t(vi) = rhotheta*theta_t(vi-1) + epsilon_theta(vi);
            g_t(vi) = rhog*g_t(vi-1) + epsilon_g(vi);
        end;
            %
            for ni=1:nper
            %
            % Create vectors of endogenous variables
            mutsim(ni)= -log(mfun(exp(theta_t(ni)),exp(g_t(ni)),psi));
            tautsim(ni) = taufun(exp(theta_t(ni)),exp(g_t(ni)),omega);
            Htsim(ni) = hfun(exp(theta_t(ni)),exp(g_t(ni)),phi);
            lambdatsim(ni) = lambdafun(exp(theta_t(ni)),exp(g_t(ni)),lam);
            Xtsim(ni) = delta*K;
            Ytsim(ni) = exp(theta_t(ni))*Kt*Htsim(ni)^alpha;  
            YNtsim(ni) = Ytsim(ni)-Xtsim(ni);
            Gtsim(ni) = exp(g_t(ni))*G0;
            C1tsim(ni) = ((Ytsim(ni)-Xtsim(ni)-Gtsim(ni))*beta*(a/(1-a))*exp(-mutsim(ni)))/...
                (1+beta*(a/(1-a))*exp(-mutsim(ni)));
            C2tsim(ni) = (Ytsim(ni)-Xtsim(ni)-Gtsim(ni))/(1+beta*(a/(1-a))*exp(-mutsim(ni)));
            C2tinvsim(ni) = 1/C2tsim(ni);
            Ressim(ni) = Ytsim(ni)-Gtsim(ni)-C1tsim(ni)-C2tsim(ni)-Xtsim(ni);
            RLtsim(ni) = rlquad6(exp(theta_t(ni)),exp(g_t(ni)),phi,psi);
            %
            if ni ==1;
                Ptsim(ni) = M0/C1tsim(ni);
                Mtsim(ni) = exp(mutsim(ni))*M0;
                inflationsim(ni) = log(Ptsim(ni)/P0);
                Dtsim(ni) = BLt0*RLt0 + Gtsim(ni) -...
                    tautsim(ni)*YNtsim(ni) - (exp(mutsim(ni))-1)*M0/Ptsim(ni);
                DYratiosim(ni)=Dtsim(ni)/YNtsim(ni);
            else
                Ptsim(ni) = Mtsim(ni-1)/C1tsim(ni);
                Mtsim(ni) = exp(mutsim(ni))*Mtsim(ni-1);
                inflationsim(ni) = log(Ptsim(ni)/Ptsim(ni-1));
                Dtsim(ni) = BLt0*RLtsim(ni-1) + Gtsim(ni) -...
                    tautsim(ni)*YNtsim(ni)-(exp(mutsim(ni))-1)*Mtsim(ni-1)/Ptsim(ni);
                DYratiosim(ni)=Dtsim(ni)/YNtsim(ni);
            end;
        end;                
    %
    % Compute Hodrick Prescott Filter of logged values
    % ------------------------------------------------
    % Program works by reading in a matrix and a scalar.
    hpscalar = 1600; 
    % Use logged variables in HP Filter.
    % Money growth and inflation are already logged.
    %
    mutfil = hpfilter(mutsim', hpscalar);
    Mtfil = hpfilter(Mtsim',hpscalar);
    tautfil = hpfilter(log(tautsim'),hpscalar);
    Htfil = hpfilter(log(Htsim'),hpscalar);
    lambdatfil = hpfilter(log(lambdatsim'),hpscalar);
    Ytfil = hpfilter(log(Ytsim'),hpscalar);
    YNtfil = hpfilter(log(YNtsim'),hpscalar);
    Gtfil = hpfilter(log(Gtsim'),hpscalar);
    C1tfil = hpfilter(log(C1tsim'),hpscalar);
    C2tfil = hpfilter(log(C2tsim'),hpscalar);
    C2tinvfil = hpfilter(log(C2tinvsim'),hpscalar);
    RLtfil = hpfilter(log(RLtsim'),hpscalar);
    Dtfil = hpfilter(log(Dtsim'),hpscalar);
    Ptfil = hpfilter(log(Ptsim'),hpscalar);
    inflationfil = hpfilter(inflationsim',hpscalar);
    %utilityfil = hpfilter(utilitysim',hpscalar);
    techfil = hpfilter(theta_t',hpscalar);
    gfil = hpfilter(g_t',hpscalar);
    %
    % Compute Statistics
    % ------------------
    % Means and standard deviations
    %
    mi=1;
    %
    % Collect means and standard deviations across each simulation.
    % The -fil matrices above have variables stacked in columns.
    % Compute statistics on the columns, not the rows.
    %
    mutstat(mi,:) = [mean(mutfil(:,mi)), std(mutfil(:,mi))*100];
    Mtstat(mi,:) = [mean(Mtfil(:,mi)), std(Mtfil(:,mi))*100];
    tautstat(mi,:) = [mean(tautfil(:,mi)), std(tautfil(:,mi))*100];
    Htstat(mi,:) = [mean(Htfil(:,mi)), std(Htfil(:,mi))*100];
    lambdatstat(mi,:) = [mean(lambdatfil(:,mi)), std(lambdatfil(:,mi))*100];
    Ytstat(mi,:) = [mean(Ytfil(:,mi)), std(Ytfil(:,mi))*100];
    YNtstat(mi,:) = [mean(YNtfil(:,mi)), std(YNtfil(:,mi))*100];
    Gtstat(mi,:) = [mean(Gtfil(:,mi)), std(Gtfil(:,mi))*100];
    C1tstat(mi,:) = [mean(C1tfil(:,mi)), std(C1tfil(:,mi))*100];
    C2tstat(mi,:) = [mean(C2tfil(:,mi)), std(C2tfil(:,mi))*100];
    C2tinvstat(mi,:) = [mean(C2tinvfil(:,mi)), std(C2tinvfil(:,mi))*100];
    RLtstat(mi,:) = [mean(RLtfil(:,mi)), std(RLtfil(:,mi))*100];
    Dtstat(mi,:) = [mean(Dtfil(:,mi)), std(Dtfil(:,mi))*100];
    Ptstat(mi,:) = [mean(Ptfil(:,mi)), std(Ptfil(:,mi))*100];
    inflationstat(mi,:) = [mean(inflationfil(:,mi)), std(inflationfil(:,mi))*100];
    %utilitystat(mi,:) = [mean(utilityfil(:,mi)), std(utilityfil(:,mi))*100];
    %
    % Compute overall means and standard deviations for entire sample
    %
    mutstd = mean(mutstat(:,2));
    Mtstd = mean(Mtstat(:,2));
    tautstd = mean(tautstat(:,2));
    Htstd = mean(Htstat(:,2));
    lambdatstd = mean(lambdatstat(:,2));
    Ytstd = mean(Ytstat(:,2));
    YNtstd = mean(YNtstat(:,2));
    Gtstd = mean(Gtstat(:,2));
    C1tstd = mean(C1tstat(:,2));
    C2tstd = mean(C2tstat(:,2));
    RLtstd = mean(RLtstat(:,2));
    Dtstd = mean(Dtstat(:,2));
    Ptstd = mean(Ptstat(:,2));
    inflationstd = mean(inflationstat(:,2));
    %utilitystd = mean(utilitystat(:,2));
    %
    stdvector=[Ytstd, C1tstd, C2tstd, Htstd, lambdatstd, Gtstd, Ptstd, inflationstd, RLtstd, Dtstd, mutstd, tautstd];
    stddevss(:,si)=stdvector';
    % Compute Cross-Correlations
    % --------------------------
    % A(:,j) = the j-th column of A
    % A(i,:) = the i-th row of A
    %
    % cross correlation with output
    cross1(:,1)=corrcoef(Ytfil,Ytfil,4);
    cross1(:,2)=corrcoef(Ytfil,C1tfil,4);
    cross1(:,3)=corrcoef(Ytfil,C2tfil,4);
    cross1(:,4)=corrcoef(Ytfil,Htfil,4);
    cross1(:,5)=corrcoef(Ytfil,lambdatfil,4);
    cross1(:,6)=corrcoef(Ytfil,Gtfil,4);
    cross1(:,7)=corrcoef(Ytfil,Ptfil,4);
    cross1(:,8)=corrcoef(Ytfil,inflationfil,4);
    cross1(:,9)=corrcoef(Ytfil,RLtfil,4);
    cross1(:,10)=corrcoef(Ytfil,Dtfil,4);
    cross1(:,11)=corrcoef(Ytfil,mutfil,4);
    cross1(:,12)=corrcoef(Ytfil,tautfil,4);
    %
    cross=cross1';
    %
    % create the matrix of cross correlations with output. Each matrix
    % below stores consecutive outcomes of the same column vector.
    crossc1(:,si)=cross(:,1);
    crossc2(:,si)=cross(:,2);
    crossc3(:,si)=cross(:,3);
    crossc4(:,si)=cross(:,4);
    crossc5(:,si)=cross(:,5);
    crossc6(:,si)=cross(:,6);
    crossc7(:,si)=cross(:,7);
    crossc8(:,si)=cross(:,8);
    crossc9(:,si)=cross(:,9);
    %
    % cross correlation with money growth
    crossmut(1,:)=corrcoef(mutfil,Ytfil,1)';
    crossmut(2,:)=corrcoef(mutfil,C1tfil,1)';
    crossmut(3,:)=corrcoef(mutfil,C2tfil,1)';
    crossmut(4,:)=corrcoef(mutfil,Htfil,1)';
    crossmut(5,:)=corrcoef(mutfil,lambdatfil,1)';
    crossmut(6,:)=corrcoef(mutfil,Gtfil,1)';
    crossmut(7,:)=corrcoef(mutfil,Ptfil,1)';
    crossmut(8,:)=corrcoef(mutfil,inflationfil,1)';
    crossmut(9,:)=corrcoef(mutfil,RLtfil,1)';
    crossmut(10,:)=corrcoef(mutfil,Dtfil,1)';
    crossmut(11,:)=corrcoef(mutfil,mutfil,1)';
    crossmut(12,:)=corrcoef(mutfil,tautfil,1)';
    %
    % cross correlation with tax rate
    crosstaut(1,:)=corrcoef(tautfil,Ytfil,1)';
    crosstaut(2,:)=corrcoef(tautfil,C1tfil,1)';
    crosstaut(3,:)=corrcoef(tautfil,C2tfil,1)';
    crosstaut(4,:)=corrcoef(tautfil,Htfil,1)';
    crosstaut(5,:)=corrcoef(tautfil,lambdatfil,1)';
    crosstaut(6,:)=corrcoef(tautfil,Gtfil,1)';
    crosstaut(7,:)=corrcoef(tautfil,Ptfil,1)';
    crosstaut(8,:)=corrcoef(tautfil,inflationfil,1)';
    crosstaut(9,:)=corrcoef(tautfil,RLtfil,1)';
    crosstaut(10,:)=corrcoef(tautfil,Dtfil,1)';
    crosstaut(11,:)=corrcoef(tautfil,mutfil,1)';
    crosstaut(12,:)=corrcoef(tautfil,tautfil,1)';
    %
    % cross correlation with multiplier
    crosslambdat(1,:)=corrcoef(lambdatfil,Ytfil,1)';
    crosslambdat(2,:)=corrcoef(lambdatfil,C1tfil,1)';
    crosslambdat(3,:)=corrcoef(lambdatfil,C2tfil,1)';
    crosslambdat(4,:)=corrcoef(lambdatfil,Htfil,1)';
    crosslambdat(5,:)=corrcoef(lambdatfil,lambdatfil,1)';
    crosslambdat(6,:)=corrcoef(lambdatfil,Gtfil,1)';
    crosslambdat(7,:)=corrcoef(lambdatfil,Ptfil,1)';
    crosslambdat(8,:)=corrcoef(lambdatfil,inflationfil,1)';
    crosslambdat(9,:)=corrcoef(lambdatfil,RLtfil,1)';
    crosslambdat(10,:)=corrcoef(lambdatfil,Dtfil,1)';
    crosslambdat(11,:)=corrcoef(lambdatfil,mutfil,1)';
    crosslambdat(12,:)=corrcoef(lambdatfil,tautfil,1)';
    %
    % cross correlation with technology shock
    crosstech(1,:)=corrcoef(techfil,Ytfil,1)';
    crosstech(2,:)=corrcoef(techfil,C1tfil,1)';
    crosstech(3,:)=corrcoef(techfil,C2tfil,1)';
    crosstech(4,:)=corrcoef(techfil,Htfil,1)';
    crosstech(5,:)=corrcoef(techfil,lambdatfil,1)';
    crosstech(6,:)=corrcoef(techfil,Gtfil,1)';
    crosstech(7,:)=corrcoef(techfil,Ptfil,1)';
    crosstech(8,:)=corrcoef(techfil,inflationfil,1)';
    crosstech(9,:)=corrcoef(techfil,RLtfil,1)';
    crosstech(10,:)=corrcoef(techfil,Dtfil,1)';
    crosstech(11,:)=corrcoef(techfil,mutfil,1)';
    crosstech(12,:)=corrcoef(techfil,tautfil,1)';
    %
    % cross correlation with government spending shock
    crossg(1,:)=corrcoef(gfil,Ytfil,1)';
    crossg(2,:)=corrcoef(gfil,C1tfil,1)';
    crossg(3,:)=corrcoef(gfil,C2tfil,1)';
    crossg(4,:)=corrcoef(gfil,Htfil,1)';
    crossg(5,:)=corrcoef(gfil,lambdatfil,1)';
    crossg(6,:)=corrcoef(gfil,Gtfil,1)';
    crossg(7,:)=corrcoef(gfil,Ptfil,1)';
    crossg(8,:)=corrcoef(gfil,inflationfil,1)';
    crossg(9,:)=corrcoef(gfil,RLtfil,1)';
    crossg(10,:)=corrcoef(gfil,Dtfil,1)';
    crossg(11,:)=corrcoef(gfil,mutfil,1)';
    crossg(12,:)=corrcoef(gfil,tautfil,1)';
    %
    crosscmut(:,si)=crossmut(:,2);
    crossctaut(:,si)=crosstaut(:,2);
    crossclambdat(:,si)=crosslambdat(:,2);
    crossctech(:,si)=crosstech(:,2);
    crosscg(:,si)=crossg(:,2);
end;
%
% Take the average of the simulations
% A(:,j) = the j-th column of A
% A(i,:) = the i-th row of A
for oi=1:length(stdvector)
    stddevoutput(oi,1)=mean(stddevss(oi,:));
    cc1(oi,1)=mean(crossc1(oi,:));
    cc2(oi,1)=mean(crossc2(oi,:));
    cc3(oi,1)=mean(crossc3(oi,:));
    cc4(oi,1)=mean(crossc4(oi,:));
    cc5(oi,1)=mean(crossc5(oi,:));
    cc6(oi,1)=mean(crossc6(oi,:));
    cc7(oi,1)=mean(crossc7(oi,:));
    cc8(oi,1)=mean(crossc8(oi,:));
    cc9(oi,1)=mean(crossc9(oi,:));
    mutcc(oi,1)=mean(crosscmut(oi,:));
    tautcc(oi,1)=mean(crossctaut(oi,:));
    lambdatcc(oi,1)=mean(crossclambdat(oi,:));
    techcc(oi,1)=mean(crossctech(oi,:));
    gcc(oi,1)=mean(crosscg(oi,:));
end;
%
% Display output
   
   stddevoutput
   cc1
   cc2
   cc3
   cc4
   cc5
   cc6
   cc7
   cc8
   cc9
   mutcc
   tautcc
   lambdatcc
   techcc
   gcc
end;
if utilitycalc
    %
    %
    % Outer loop defines the number of simulations. Inner loop defines 
    % the number of periods for each simulation.
    % Equal to nper as defined above for impulse response functions.
    %
    load shockthetaedit;
    load shockgedit;
    %
    for si = 1:nsim
        %
        % Inner loop defines the number of periods for each simulation.
        % Pull the row of each shock to run consistent simulations across
        % different debt specifications.
        %
        theta_t=thetamatrix(si,:);
        g_t=gmatrix(si,:);
        %
        for ni=1:nper
            %
            %
            mutsim(ni)= -log(mfun(exp(theta_t(ni)),exp(g_t(ni)),psi));
            tautsim(ni) = taufun(exp(theta_t(ni)),exp(g_t(ni)),omega);
            Htsim(ni) = hfun(exp(theta_t(ni)),exp(g_t(ni)),phi);
            lambdatsim(ni) = lambdafun(exp(theta_t(ni)),exp(g_t(ni)),lam);
            Xtsim(ni) = delta*K;
            Ytsim(ni) = exp(theta_t(ni))*Kt*Htsim(ni)^alpha;  
            YNtsim(ni) = Ytsim(ni)-Xtsim(ni);
            Gtsim(ni) = exp(g_t(ni))*G0;
            C1tsim(ni) = ((Ytsim(ni)-Xtsim(ni)-Gtsim(ni))*beta*(a/(1-a))*exp(-mutsim(ni)))/...
                (1+beta*(a/(1-a))*exp(-mutsim(ni)));
            C2tsim(ni) = (Ytsim(ni)-Xtsim(ni)-Gtsim(ni))/(1+beta*(a/(1-a))*exp(-mutsim(ni)));
            C2tinvsim(ni) = 1/C2tsim(ni);
            Ressim(ni) = Ytsim(ni)-Gtsim(ni)-C1tsim(ni)-C2tsim(ni)-Xtsim(ni);
            %
            % The utility value
            % Make per period adjustments for steady state differences in levels. .002
            % for high debt case and .001 for low debt case.
            if D_Y==2*.049;
                utilitysim(ni) = (beta^ni)*(a*log(C1tsim(ni)) + (1-a)*log(C2tsim(ni))...
                    -gamma*Htsim(ni));
            elseif D_Y==0.49;
                utilitysim(ni) = (beta^ni)*(a*log(C1tsim(ni)) + (1-a)*log(C2tsim(ni))...
                    -gamma*Htsim(ni));
            else
                utilitysim(ni) = (beta^ni)*(a*log(C1tsim(ni)) + (1-a)*log(C2tsim(ni))...
                    -gamma*Htsim(ni));
            end;
            utilitysim1(ni)=(a*log(C1tsim(ni)) + (1-a)*log(C2tsim(ni))...
                    -gamma*Htsim(ni));
            % The interest rate terms for period 1 depend on the actual 
            % realizations of the shocks and expectations of future variables.  
            % 
            RLtsim(ni) = rlquad6(exp(theta_t(ni)),exp(g_t(ni)),phi,psi);
            %
            % The budget constraint leaves debt as the residual
            %
            if ni ==1;
                Ptsim(ni) = M0/C1tsim(ni);
                Mtsim(ni) = exp(mutsim(ni))*M0;
                %
                inflationsim(ni) = log(Ptsim(ni)/P0);
                Dtsim(ni) = BLt0*RLt0 + Gtsim(ni) -...
                    tautsim(ni)*YNtsim(ni) - (exp(mutsim(ni))-1)*M0/Ptsim(ni);
            else
                Ptsim(ni) = Mtsim(ni-1)/C1tsim(ni);
                Mtsim(ni) = exp(mutsim(ni))*Mtsim(ni-1);
                %
                inflationsim(ni) = log(Ptsim(ni)/Ptsim(ni-1));
                Dtsim(ni) = BLt0*RLtsim(ni-1) + Gtsim(ni) - tautsim(ni)*YNtsim(ni) -...
                    (exp(mutsim(ni))-1)*Mtsim(ni-1)/Ptsim(ni);
            end;
        end;                
        %
        % Sum up utility over each simulation
        utility=sum(utilitysim);
        %end;
    utility';
    utilitymean(si,:)=(utility);
    utility1=mean(utilitymean);
    if(printer),fprintf(1,'\n it %g, avgutility %g,',si, utility1),end;
end;
end;    
        


