% Sub-Matlab program for :
% Gapen, Michael T. and Thomas F. Cosimano, 
% "Sovling Ramsey Problems with Nonlinear Projection Methods," 
% Studies in Nonlinear Dynamics and Econometrics, Vol. 9, Spring
% 2005.
%
% function R = projectionSNDE2(coeffvec,xgrid)
% Purpose: To create the system of residual equations.
%
% Opening Commands
% ==============================================================================

function R = projectionSNDE2(coeffvec,xgrid);

global alpha beta a rhotheta rhog sigmatheta sigmag thetamax thetamin gmax gmin
global thetaord gord tnod gnod G0 theta0 Y g0 gamma Ynet
global C1 C2 M mu G_Y C1_Y H Kt D_Y X

% ==============================================================================
%
format long;
% Section 1.0 Break out the coeffecient vector
% ============================================
%
% The coefficient vector is loaded into projection.m as a combined vector, but 
% must be split out into the original vectors in order to create the polynomials
%
lam=coeffvec(1:(thetaord+1)*(gord+1));
phi=coeffvec(((thetaord+1)*(gord+1)+1):(2*(thetaord+1)*(gord+1)));
psi=coeffvec((2*(thetaord+1)*(gord+1)+1):(3*(thetaord+1)*(gord+1)));
omega=coeffvec((3*(thetaord+1)*(gord+1)+1):(4*(thetaord+1)*(gord+1)));
%
%
% Each residual vector will contain one equation for each combination of theta and g
%
dimgrid	= length(xgrid);
% 
% These are for the residual functions.  To match the first order conditions of the problem,
% name each according to the variable being optimized over (i.e. dF/dx=0, dF/dy=0) where tau,
% mu, and H refer to taxes, money, and labor, respectively.
%
residvec_lambda = zeros(dimgrid,1);
residvec_tau    = zeros(dimgrid,1);
residvec_mu     = zeros(dimgrid,1);
residvec_H      = zeros(dimgrid,1);
%
% Section 2.0 Specify remaining debt paramaters and begin loop
% ============================================================
% 
BL_D=1.0;

% Section 3.0 Create the Residual functions
% =========================================
%
% Projection methods assume that the problem lives on a compact domain.  While in theory,
% the problem specified has an unbounded ergodic set, in practice we specify a compact set
% of theta and g (theta,g) pairs over which we try to solve the residual functions.  The 
% choice for this set is represented by thetamin, thetamax, gmin, and gmax.  Since these are
% exogenously defined stochastic functions, the choice of the pairs is straightforward.
% 
% The following solution procedure is called the Galerkin and Orthogonal Collocation method. 
% The Galerkin method fixes the coefficients of the residual functions and imposes 
% (thetaord+1)*(gord+1) moment conditions.  This is done when we approximate the true
% function with Chebyshev Polynomials.  Since the solution to the residual functions involves
% computing the integral, which is unknown, the collocation method proceeds by choosing 
% coefficients so that the residual functions are zero at a finite set of points. 

% The solution process chooses thetagrid capital stocks and ggrid government spending shocks
% and then searches for coefficients such that the residual functions equal zero at each 
% gridpoint.
%
% Thus, the following loop runs from 1:dimgrid, which is the length/size of xgrid.  xgrid
% contains all the possible combinations of theta and g.  Three residual functions will be 
% constructed for each possible combination.
%
% Calculate the expectation terms using Gauss-Hermite Quadrature
% --------------------------------------------------------------
% The equations below require the evaluation of expectations operators.  This is done using
% gauss-hermite quadrature.  The expectations are static in the sense that the value of the
% expectation is the same for each grid space combination.  Therefore, the program only
% needs to calculate these once outside the main loop and the loop will read these in as if
% they were constants.
%
ERL = rlquad5(phi,psi);
Etau1 = tauquad5(phi,psi,omega);
EC2t = c2tquad5(phi,psi);
Emu1 = muquad5(phi);
Emu3 = muquad6(psi);
%
% The initial values of debt are based on the exogenous specification
% for D/Y and the initial values of theta and g.  
BL0 = BL_D*D_Y*(exp(theta0)*Kt*(hfun(exp(theta0),exp(g0),phi))^alpha-X);
%
% P(t-1) used in the computation of nominal debt above
P0 = (M/(exp(theta0)*Kt*(hfun(exp(theta0),exp(g0),phi))^alpha-X-exp(g0)*G0))*...
    (1+beta*(a/(1-a))*mfun(exp(theta0),exp(g0),psi))/...
    (beta*(a/(1-a)));
%
dim=0;
for i   = 1:dimgrid;
   % Scale up from [-1,1] to [a,b]
   % -----------------------------
   % The grid over the possible realizations of the productivity and spending
   % shocks in xgrid are the zeros of the Chebyshev polynomial. These are formed over the 
   % interval [-1,1] and must be scaled up to the [a,b] interval for use in 
   % the residual functions.  The Chebyshev polynomials and residual functions
   % will be created based on these values.
   thetaup=exp(scalup(xgrid(i,1),thetamin,thetamax));
   gup=exp(scalup(xgrid(i,2),gmin,gmax));
   %
   % The polynomials  
   % ---------------
   lambdapol = lambdafun(thetaup,gup,lam);
   moneypol = mfun(thetaup,gup,psi);
   laborpol = hfun(thetaup,gup,phi);
   taxpol = taufun(thetaup,gup,omega);
   %
   Elambda=lambdaquad(thetaup,gup,phi,psi,lam);
   %
   YG=thetaup*Kt*(laborpol)^alpha-X-gup*G0;
   %
   % Calculate (1+(1-alpha)/alpha*(Yt-X-Gt)/Yt)^(-1)
   % This comes from the marginal product of labor times the change in labor supply.
   % Thus, call it MPL*deltaH or MPLH
   %
   MPLH = 1/(1 + ((1-alpha)/alpha)*(YG/(thetaup*Kt*(laborpol)^alpha)));
   %
   % Create the tax residual function
   % --------------------------------
   % The first residual function is from the tax derivative
   R2A = 1-(1-a)*(1+beta*(a/(1-a))*moneypol)*(1-taxpol);
   R2B = lambdapol;
   R2C = (1-taxpol)*YG*...
       (1+((1-alpha)/alpha)*(thetaup*Kt*(laborpol)^alpha-X)/(thetaup*Kt*(laborpol)^alpha));
   R2D = BL0*ERL*Etau1*EC2t*((1-taxpol)*...
        (1 + ((1-alpha)/alpha)*(YG/(thetaup*Kt*(laborpol)^alpha))));
   R2E = -BL0*ERL;
   %
   residvec_tau(i) = R2A - R2B*(R2C + R2D + R2E);
   %       
   % Create the money residual function
   % ----------------------------------
   % First some preliminary calculations.
   % The cash good, credit good, and output
   %
   CON1 = (YG*beta*(a/(1-a))*moneypol)/(1+beta*(a/(1-a))*moneypol);
   CON2 = YG/(1+beta*(a/(1-a))*moneypol);
   Y1=thetaup*Kt*(laborpol)^alpha;    % Yt
   Y1net = Y1 - X;                    % Ynet  
   ERL1=rlquad6(thetaup,gup,phi,psi); % RLt
   MP = CON1;                         % Mt/Pt  Use CIA constraint.
   M1P = 1/moneypol*MP;               % M(t+1)/Pt
   BL1=BL0;                             % BL(t+1)
   %
   R3A = -a + CON1/Y1*MPLH*((1-alpha)/alpha);    
   R3B = (1-a)*(1-taxpol)*beta*(a/(1-a))*moneypol*MPLH;
   R3C = lambdapol;   
   R3D = taxpol*CON1*MPLH*(1/alpha);
   R3E = -(gup*G0 + taxpol*X + BL0*ERL);
   R3F = CON1/Y1*((1-alpha)/alpha)*MPLH;
   R3G = -CON1;
   R3H = moneypol;     
   R3J = BL0*ERL*EC2t*Emu1;
   %
   LHS=R3A + R3B - R3C*(R3D+R3E*R3F+R3G+R3H*R3J);
   %
   R3Q = 0; %Comes from transversality conditions
   % 
   residvec_mu(i) = LHS+R3Q;
   %
   % Create the labor residual function
   % ----------------------------------
   % The third residual function is from the labor derivative
   % It does not contain any expectation terms.
   %
   R4A = gamma*YG;
   R4B = (1+beta*(a/(1-a))*moneypol);
   R4C = (1-a)*(1-taxpol)*alpha*thetaup*Kt*(laborpol)^(alpha-1);
   %   
   residvec_H(i) = R4A/R4B - R4C;
   %
   % The Government budget constraint
   % --------------------------------
   % The fourth residual function is for lambda
   %
   G1=gup*G0;
   residvec_lambda(i) = (BL0*ERL-BL1+MP-taxpol*Y1net+G1-M1P);
end;
%
% Section 4.0 Stack the system of residual functions 
% ==================================================
%
% Begin with the tax derivative, then money, then labor
%
R=[residvec_lambda ; residvec_tau ; residvec_mu ; residvec_H];



