% Sub-Matlab program for :
% Gapen, Michael T. and Thomas F. Cosimano, 
% "Sovling Ramsey Problems with Nonlinear Projection Methods," 
% Studies in Nonlinear Dynamics and Econometrics, Vol. 9, Spring
% 2005.
%
% function real=tauquad5(phi,psi,omega);
%
% Purpose: Use Bivariate Gauss-Hermite Quadrature to evaluate 
% the expectation term in the euler condition for taxes under committment.
%
% Source:
%
% =============================================================================

function real=tauquad5(phi,psi,omega);

global alpha beta a rhotheta rhog sigmatheta sigmag thetamax thetamin gmax gmin
global thetaord gord tnod gnod G0 theta0 Y g0
global C1 C2 M mu G_Y C1_Y H Kt X


% Section 1.0 Create the Gauss-Hermite Quadrature weights and nodes
% =================================================================
%
[nodtheta wtheta]=hernodes(tnod);
[nodg wg]=hernodes(gnod);
%
% Gauss-Hermite Quadrature uses the weighting function exp{-x^2}. 
% Here, however, we have the density 
% (1./sigma*sqrt(2*pi))*exp{-y^2/2*sigma^2} and want to calculate the
% integral of fun(y)*(1./sigma*sqrt(2*pi))*exp{-y^2/2*sigma^2}
%
% Adjusting the quadrature procedure requires the following two steps.
% First a change of variables is introduced. In particular, y=sqrt(2)*sigma*x.
% This results in the weighting function (1/sigma*sqrt(2*pi))*exp{-x^2}. 
% Second, we have to multiply the integrand with the Jacobian of the 
% transformation. This gives the weighting function (1/sqrt(pi))*exp{-x^2}
% and calculating the integral of fun(sqrt(2)*sigma*x)*(1/sqrt(pi))*exp{-x^2}.
%
% Furthermore, here we have a bivariate system.  The above describes a univariate
% procedure.  For a bivariate system, the weighting function is exp(-x1^2)*exp(-x2^2).
% In addition the density functions cancel terms which leaves a (pi)^(-1) instead of 
% sqrt(pi).
%
% Scale the weights by the standard deviations
%
nodestheta=exp(sigmatheta*sqrt(2)*nodtheta);
nodesg=exp(sigmag*sqrt(2)*nodg);
%
% Associate the weights and nodes across the state space
% 
% This loop associates the weight with each realization of the nodes.  The wrgrid is a
% (gnod*tnod) x 1 vector and nrgrid is a (gnod*tnod) x 2 matrix.  The rows of the wrgrid 
% vector give the probability of getting the nrgrid pair.
%
for ai = 1:tnod;
   for di = 1:gnod;
      wrgrid((ai-1)*gnod+di)=wtheta(ai)*wg(di);
      nrgrid((ai-1)*gnod+di,:) = [nodestheta(ai) nodesg(di)];
      end;
   end;
   
  
rdimgrid = length(nrgrid);
E1=zeros(rdimgrid,1);
%
% Section 2.0 Calculate the Expected Value 
% ========================================
% 
% Note that the polynomials below are evaluated at thetax and gx, the nodes
% of the Gauss-Hermite Quadrature over [-inf, inf].  These may be different than
% the thetaup, gup values used to create the polynomials in the remaining terms.
% The extent that thetaup and gup differ from thetax and gx will determine the amount
% of nonlinearity or curvature in the system.
%
thetax=nrgrid((1:1:rdimgrid),1);
gx=nrgrid((1:1:rdimgrid),2);
% Note that the calculation below uses thetax and gx rather than exp(thetax)
% and exp(gx).  This is because thetax and gx are already defined in this fashion.
% When referring to the t-1 values of theta0 and g0, these require 
% the exponential function.
%
E1=((1+beta*(a/(1-a)).*mfun(thetax,gx,psi))./(thetax.*Kt.*...
    (hfun(thetax,gx,phi)).^alpha-X-gx.*G0)).*(1./(1-taufun(thetax,gx,omega))).*...
    ((1+((1-alpha)/alpha).*((thetax.*Kt.*(hfun(thetax,gx,phi)).^alpha-X-gx.*G0)./...
    (thetax.*Kt.*(hfun(thetax,gx,phi)).^alpha))).^(-1));
%
% Note below that the weights and scaling with pi are applied only to the expectations
% term since 1 divided by the expected value of the function is not the same as
% the expected value of 1 divided by the function due to Jenson's Inequality.
% Thus, the Gauss-Hermite quadrature is applied only to the term within the expectations
% operator and not to the entire definition of the real interest rate.
%

real=(1/pi)*wrgrid*E1;
  